/*============================================================================
  File:     ShrinkFragmentationDemo.sql

  Summary:  This script shows how data file
			shrink can cause massive index
			fragmentation

  SQL Server Versions: 2005 onwards
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  (c) 2013, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE [master];
GO

IF DATABASEPROPERTYEX (N'DBMaint2008', N'Version') > 0
BEGIN
	ALTER DATABASE [DBMaint2008] SET SINGLE_USER
		WITH ROLLBACK IMMEDIATE;
	DROP DATABASE [DBMaint2008];
END
GO


CREATE DATABASE [DBMaint2008];
GO
USE [DBMaint2008];
GO

SET NOCOUNT ON;
GO

-- Create the 10MB filler table at the
-- 'front' of the data file
CREATE TABLE [FillerTable] (
	[c1] INT IDENTITY,
	[c2] CHAR (8000) DEFAULT 'filler');
GO

-- Fill up the filler table
INSERT INTO [FillerTable] DEFAULT VALUES;
GO 1280

-- Create the production table, which
-- will be 'after' the filler table in
-- the data file
CREATE TABLE [ProdTable] (
	[c1] INT IDENTITY,
	[c2] CHAR (8000) DEFAULT 'production');
CREATE CLUSTERED INDEX [prod_cl]
	ON [ProdTable] ([c1]);
GO

INSERT INTO [ProdTable] DEFAULT VALUES;
GO 1280

-- check the fragmentation of the production table
SELECT
	[avg_fragmentation_in_percent]
FROM sys.dm_db_index_physical_stats (
	DB_ID (N'DBMaint2008'),
	OBJECT_ID (N'ProdTable'),
	1,
	NULL,
	N'LIMITED');
GO

-- drop the filler table, creating 10MB
-- of free space at the 'front' of the
-- data file
DROP TABLE [FillerTable];
GO

-- shrink the database
DBCC SHRINKDATABASE (N'DBMaint2008');
GO

-- check the index fragmentation again
SELECT
	[avg_fragmentation_in_percent]
FROM sys.dm_db_index_physical_stats (
	DB_ID ('DBMaint2008'),
	OBJECT_ID ('ProdTable'),
	1,
	NULL,
	N'LIMITED');
GO
